'
' Example script for 2BrightSparks SyncBackPro V6 that lets you treat 
' all drives as a single drive.
'
' SBLang=VBScript
'
' http://www.2BrightSparks.com/
' May 2010
'
Dim FSO, Drives, IsMainThread, DebugLog

'
' This is a location script 
'
Function Description(ScriptType)
  Description = "All Drives Location"
  ScriptType = 8
End Function

'
' This is called when the user adds the script to SyncBack
'
Function Install(Interactive)
'  If Interactive Then
'    SBSystem.Say "Installed"
'  End If
End Function

'
' This is called when the user removes the script to SyncBack
'
Sub Uninstall()
'  SBSystem.Say "Uninstalled"
End Sub

'
' The script is being exported
'
Function FilesToExport(Interactive, Cnt)
'  If Cnt = 0 Then
'    FilesToExport = "D:\temp\junction.exe"
'  ELseIf Cnt = 1 Then
'    FilesToExport = "C:\temp\test.txt"
'  Else
'    FilesToExport = ""
'  End If
End Function

'
' Output a debug string
'
Sub DebugOut(Msg)
  if Not (DebugLog is nothing) then
    DebugLog.WriteLine(Msg)
  End If
End Sub

'
' Given a path like \C\Folder\Filename.txt then we return C:\Folder\Filename.txt
'
Function ExtractFilename(FullPath)
  ExtractFilename = ""

  if FullPath = "\" or FullPath = "" then
    '
    ' The root cannot be used
    '
    Exit Function
  else
    '
    ' Return the filename
    '
    DriveLetter = Mid(FullPath, 2, 1)
    if DriveLetter = "" then
      Exit Function
    End If

    DrivePath = Mid(FullPath, 3, Len(FullPath) - 2)
    If DrivePath = "" then
      DrivePath = "\"
    End If

    ExtractFilename = DriveLetter & ":" & DrivePath
  end if
End Function

'
' Return an error message if we cannot connect
'
Function LocConnect(MainThread)
  LocConnect = ""

  IsMainThread = MainThread

  Set FSO = CreateObject("Scripting.FileSystemObject")
  Set Drives = FSO.Drives

  If Not FSO.FolderExists("C:\temp\") Then
    FSO.CreateFolder("C:\temp\")
  End If  
  
  Set Folder = FSO.GetFolder("C:\temp\")
  If Not IsMainThread Then
    Set DebugLog = Folder.CreateTextFile("debug.txt", True, True)
  Else
    Set DebugLog = Folder.CreateTextFile("debug_main.txt", True, True)
  End If

  DebugOut "---Connected: " & CStr(Now)
End Function

'
' Return an error message if we cannot disconnect
'
Function LocDisconnect()
  LocDisconnect = ""

  Set Drives = nothing
  Set FSO = nothing

  if Not (DebugLog is nothing) then
    DebugOut "---Disconnected: " & CStr(Now)
    DebugLog.WriteBlankLines(1)
    DebugLog.Close
    Set DebugLog = nothing
  End If
End Function

'
' Return an error message if we cannot prepare it
'
Function LocScanPrepare()
  LocScanPrepare = ""

  DebugOut "---LocScanPrepare"
End Function

'
' Return an error message if we are not connected
'
Function LocConnected()
  DebugOut "---LocConnected"

  LocConnected = ""
End Function

'
' Return an error message if we cannot reconnect
'
Function LocReconnect()
  DebugOut "---LocReconnect"

  LocReconnect = ""
End Function

'
' Return an error message if the directory does not exist. The full directory
' is passed (which always includes a trailing backslash).
'
Function LocDirExists(FullPath)
  DebugOut "---LocDirExists:" & FullPath

  FullPath = ExtractFilename(FullPath)
  If FullPath = "" Then
    LocDirExists = ""
  ElseIf FSO.FolderExists(FullPath) Then
    LocDirExists = ""
  Else
    LocDirExists = "Folder does not exist"
  End If
End Function

'
' Return an error message if the file does not exist. The full filename
' is passed.
'
Function LocFileExists(FullPath)
  DebugOut "---LocFileExists:" & FullPath

  FullPath = ExtractFilename(FullPath)
  If FullPath = "" then
    LocFileExists = "File does not exist"
  ElseIf FSO.FileExists(FullPath) Then
    LocFileExists = ""
  Else
    LocFileExists = "File does not exist"
  End If
End Function

'
' Create a directory. A full path is passed, including trailing backslash.
' Returns an error message on failure (if the directory already exists then
' no error should be returned).
'
Function LocMakeDir(FullPath)
  DebugOut "---LocMakeDir:" & FullPath

  LocMakeDir = ""

  FullPath = ExtractFilename(FullPath)
  If FullPath = "" Then
    Exit Function
  ElseIf Not FSO.FolderExists(FullPath) Then
    FSO.CreateFolder(FullPath)
  End If
End Function

'
' Delete a directory. A full path is passed, including trailing backslash.
' Returns an error message on failure. No error message should be returned if
' the directory does not exist. A directory should not be deleted by the script
' unless it is empty, i.e. has no files or directories in it.
'
Function LocRemoveDir(FullPath)
  DebugOut "---LocRemoveDir:" & FullPath

  LocRemoveDir = ""

  FullPath = ExtractFilename(FullPath)

  If FullPath = "" Then
    Exit Function
  ElseIf FSO.FolderExists(FullPath) Then
    Set Folder = FSO.GetFolder(FullPath)

    If Folder.Files.Count > 0 Then
      LocRemoveDir = "Directory contains files"
      Exit Function
    End If

    If Folder.SubFolders.Count > 0 Then
      LocRemoveDir = "Directory contains directories"
      Exit Function
    End If

	Folder.Delete True
    Set Folder = nothing
  End If
End Function

'
' Rename a folder. Return an error message on failure.
'
Function LocRenameDir(FromPath, ToPath)
  DebugOut "---LocRenameDir:" & FromPath & "*" & ToPath

  FullFromPath = ExtractFilename(FromPath)
  FullToPath = ExtractFilename(ToPath)

  If FullFromPath = "" Then
    LocRenameDir = "Root directory cannot be moved"
    Exit Function
  ElseIf FullToPath = "" Then
    LocRenameDir = "Cannot move to root directory"
    Exit Function
  End If

  If Not FSO.FolderExists(FullFromPath) Then
    LocRenameDir = "Source directory does not exist"
    Exit Function
  End If

  If FSO.FolderExists(FullToPath) Then
    If SBSystem.SameFilenames(FullFromPath, FullToPath, TRUE) Then
      LocRenameDir = "To destination directory already exists"
      Exit Function
    End if
  End If

  ' Do not include trailing backslash
  If SBSystem.IsFolder(FullFromPath) Then
    FullFromPath = Left(FullFromPath, Len(FullFromPath) - 1)
  End If
  If SBSystem.IsFolder(FullToPath) Then
    FullToPath = Left(FullToPath, Len(FullToPath) - 1)
  End If

  ' Cannot rename just the case, e.g. \abc to \ABC
  ' Must instead do a two-step process
  If SBSystem.SameFilenames(FullFromPath, FullToPath, FALSE) Then
    TempFldr = FullToPath & "." & SBSystem.UniqueID
    FSO.MoveFolder FullFromPath, TempFldr
    FSO.MoveFolder TempFldr, FullToPath
  Else
    FSO.MoveFolder FullFromPath, FullToPath
  End IF
  LocRenameDir = ""
End Function

'
' Rename a file. Return an error message on failure.
'
Function LocRenameFile(FromName, ToName)
  DebugOut "---LocRenameFile:" & FromName & "*" & ToName

  FullFromName = ExtractFilename(FromName)
  FullToName = ExtractFilename(ToName)

  If Not FSO.FileExists(FullFromName) Then
    LocRenameFile = "Source file does not exist"
    Exit Function
  End If

  ' Get file attributes. Rename will give archive attribute.
  Set FileObj = FSO.GetFile(FullFromName)
  Attrs = FileObj.Attributes

  If FSO.FileExists(FullToName) Then
    If SBSystem.SameFilenames(FullFromName, FullToName, TRUE) Then
      LocRenameFile = "To destination file already exists"
      Exit Function
    End If
  End If

  FSO.MoveFile FullFromName, FullToName
  LocRenameFile = ""

  ' Set attributes
  Set FileObj = FSO.GetFile(FullToName)
  FileObj.Attributes = Attrs
End Function

'
' Delete a file. Return an error message on failure. Note that if the file does
' not exist then no error should be returned and DoesNotExist should be set
' to TRUE.
'
Function LocDeleteFile(FullPath, DoesNotExist)
  DebugOut "---LocDeleteFile:" & FullPath

  LocDeleteFile = ""

  FullPath = ExtractFilename(FullPath)

  DoesNotExist = Not FSO.FileExists(FullPath)

  If Not DoesNotExist Then
    FSO.DeleteFile FullPath, True
  End If
End Function

'
' Calculate CRC32 hash value of a file
'
Function LocCRC32(FullPath, CRC32)
  DebugOut "---LocCRC32:" & FullPath
  
  FullPath = ExtractFilename(FullPath)
  CRC32 = SBSystem.CRC32(FullPath)
  LocCRC32 = ""
End Function

'
' Set the last modification date & time of a file. ModDateTime is the date &
' time in the local timezone. On failure an error message should be returned.
'
Function LocSetModDateTime(FullPath, ModDateTime)
  DebugOut "---LocSetModDateTime:" & FullPath & "," & ModDateTime

  FullPath = ExtractFilename(FullPath)
  LocSetModDateTime = SBSystem.SetLastModDateTime(FullPath, ModDateTime)
End Function

'
' Set the creation date & time of a file or folder. CreateDateTime is the date &
' time in the local timezone. On failure an error message should be returned.
'
Function LocSetCreateDateTime(FullPath, CreateDateTime)
  DebugOut "---LocSetCreateDateTime:" & FullPath & "," & CreateDateTime

  FullPath = ExtractFilename(FullPath)
  LocSetCreateDateTime = SBSystem.SetCreateDateTime(FullPath, CreateDateTime)
End Function

'
' Get a file or folders attributes. A folder will have a trailing backslash.
' On failure return an error message.
'
Function LocGetAttributes(Filename, Attrs)
  DebugOut "---LocGetAttributes:" & Filename & "," & Attrs

  Filename = ExtractFilename(Filename)
  If Filename = "" Then
    ' Its a special folder
    LocGetAttributes = ""
    Attrs = 1 + 2 + 4 + 16 
    Exit Function
  End If

  If SBSystem.IsFolder(Filename) Then
    '
    ' A folder
    '
    If not FSO.FolderExists(Filename) Then
      LocGetAttributes = "Folder does not exist"
      Exit Function
    End If

    Set FolderObj = FSO.GetFolder(Filename)
    Attrs = FolderObj.Attributes
    LocGetAttributes = ""
  Else
    '
    ' A file
    '
    If not FSO.FileExists(Filename) Then
      LocGetAttributes = "File does not exist"
      Exit Function
    End If

    Set FileObj = FSO.GetFile(Filename)
    Attrs = FileObj.Attributes
    LocGetAttributes = ""
  End If
End Function

'
' Set a file or folders attributes. A folder will have a trailing backslash.
' On failure return an error message.
'
Function LocSetAttributes(Filename, Attrs)
  DebugOut "---LocSetAttributes:" & Filename & "," & Attrs

  Filename = ExtractFilename(Filename)
  If Filename = "" Then
    ' Its a special folder
    LocSetAttributes = "Cannot set roots attributes"
    Exit Function
  End If

  If SBSystem.IsFolder(Filename) Then
    '
    ' A folder
    '
    If not FSO.FolderExists(Filename) Then
      LocSetAttributes = "Folder does not exist"
      Exit Function
    End If

    Set FolderObj = FSO.GetFolder(Filename)
    FolderObj.Attributes = Attrs
    LocSetAttributes = ""
  Else
    '
    ' A file
    '
    If not FSO.FileExists(Filename) Then
      LocSetAttributes = "File does not exist"
      Exit Function
    End If

    Set FileObj = FSO.GetFile(Filename)
    FileObj.Attributes = Attrs
    LocSetAttributes = ""
  End If
End Function

'
' Return the number of free bytes available for file storage. If it's not
' relevant or practical then return -1.
'
'
' We cannot know because it could be any drive
'
Function LocFreeSpace()
  DebugOut "---LocFreeSpace"

  LocFreeSpace = -1  
End Function

'
' Return the abilities, i.e. what it can do and support. The values are
' a bitmask, e.g. CAN_USEATTRIBUTES + CAN_NTFSATTRIBUTES
'
' The following are valid for scripts:
'
'  CAN_COPYDIRATTRS =        2       Can the directory attributes and date & times be copied to a new directory?
'  CAN_USEATTRIBUTES =       32      Do files/folders have Windows attributes?
'  CAN_EXACTDATETIME =       64      Are file dates & times stored exactly (including milli-seconds)?
'  CAN_CHANGEATTRIBUTES =    128     Can file/folder attributes be changed?
'  CAN_CHANGEDATETIME =      256     Can file/folder date & times be changed?
'  CAN_HAVEEMPTYPATH =       1024    Can the base path be empty?
'  CAN_VERSION =             4096    Does the location support versioning?
'  CAN_NTFSATTRIBUTES =      16384   Files & folders have NTFS extended attributes?
'  CAN_WRITEONCE =           65536   Write-once, meaning cannot read from it only write to it, e.g. burn location and spanned/split zip
'  CAN_MOVE_FILES =          131072  Can files be moved?
'  CAN_USECRC32 =            262144  Can use CRC32 hashing?
'  CAN_MOVE_FOLDERS =        524288  Can folders be moved?
'
Function LocAbilities()
  ' Cannot use CAN_EXACTDATETIME because some drives may be FAT32
  ' Cannot use CAN_NTFSATTRIBUTES because some drives may be FAT32
  LocAbilities = 2 + 32 + 128 + 256 + 1024 + 4096 + 131072 + 262144 + 524288
End Function

'
' Take a file from the file-system and store it. fromFName is the full filename
' of the file to copy. toFName is where the file should ultimately be copied
' to (take note of DoSafeCopy parameter). fromAttrs are the files attributes
' (ignore them if the value is < 0). from DateTime is the last modification
' date & time of the file (in the current timezone, also ignore it if its
' value is <= 1.0). fromFileSize is the size of the file in bytes (it may be
' less than zero if its unknown). Note that its a 64-bit value. If DoSafeCopy
' is TRUE then the file should be copied to a temporary file first and not to
' toFName. The full path of the safe filename used should be returned in
' SafeFName.
'
' On failure an error message should be returned.
'
Function LocPut(fromFName, toFName, fromAttrs, fromDateTime, fromFileSize, DoSafeCopy, SafeFName)
  DebugOut "---LocPut:" & fromFName & "*" & toFName

  If Not FSO.FileExists(fromFName) Then
    LocPut = "File does not exist"
  Else
    ' Safe copy?
    If DoSafeCopy Then
      ' Note that we must return a SafeFName that we will understand when
      ' it is passed back to us (we will be asked to move the file)
      FulltoFName = ExtractFilename(toFName) & ".$$$"
      SafeFName = toFName & ".$$$"
    Else
      FulltoFName = ExtractFilename(toFName)
      SafeFName = ""
    End If

    ' Destination file read-only?
    If FSO.FileExists(FulltoFName) Then
      Set FileObj = FSO.GetFile(FulltoFName)
      Attrs = FileObj.Attributes
      If Attrs And 1 Then
        FileObj.Attributes = Attrs - 1
      End If
      Set FileObj = nothing
    End If

    ' SBSystem.UpdateFileStatus("Copying " & fromFName & "...")
    FSO.CopyFile fromFName, FulltoFName, True

    If fromAttrs >= 0 Then
      Set FileObj = FSO.GetFile(FulltoFName)
      FileObj.Attributes = fromAttrs
      Set FileObj = nothing
    End If
    
    If fromDateTime > 1.0 Then
      SBSystem.SetLastModDateTime FulltoFName, fromDateTime
    End If

    LocPut = ""
  End If
End Function

'
' Put a file onto the file-system. fromFName is the full filename of the file
' the script should copy, and toFName is the full filename of where to copy
' the file to (on file-system).
'
' On failure an error message should be returned.
'
Function LocGet(fromFName, toFName)
  DebugOut "---LocGet:" & fromFName & "*" & toFName

  FullFromName = ExtractFilename(fromFName)

  If Not FSO.FileExists(FullFromName) Then
    LocGet = "File does not exist"
  Else
    ' Read-only?
    If FSO.FileExists(toFName) Then
      Set FileObj = FSO.GetFile(toFName)
      Attrs = FileObj.Attributes
      If Attrs And 1 Then
        FileObj.Attributes = Attrs - 1
      End If
      Set FileObj = nothing
    End If

    ' SBSystem.UpdateFileStatus("Copying " & fromFName)
    FSO.CopyFile FullFromName, toFName, True
    LocGet = ""
  End If
End Function

'
' Scan a path and return all the files and folders in that path.
'
Function LocScanList(FullPath)
  LocScanList = ""

  if FullPath = "\" or FullPath = "" then
    '
    ' Return the list of drives
    '
    For Each DiskDrive in Drives
      If DiskDrive.IsReady then
        If SBLocation.AddDir(DiskDrive.DriveLetter) = False then
          Exit Function
        End If
      End If
    Next
  else
    '
    ' Return the list of folders and files
    '
    DriveLetter = Mid(FullPath, 2, 1)
    if DriveLetter = "" then
      Exit Function
    End If
    DrivePath = Mid(FullPath, 3, Len(FullPath) - 2)
    If DrivePath = "" then
      DrivePath = "\"
    End If
    Path = DriveLetter & ":" & DrivePath

    ' Return a list of sub-folders
    Set Folder = FSO.GetFolder(Path)
    Set FolderCol = Folder.SubFolders
    For Each SubFol in FolderCol
      If SBLocation.AddDirEx(SubFol.Name, SubFol.Attributes) = False then
        Exit Function
'      If not SubFol.Attributes and 1024 then
'        If SBLocation.AddDir(SubFol.Name) = False then
'          Exit Function
'        End If
      End If
    Next

    ' Return a list of files
    Set FilesCol = Folder.Files
    For Each FileItem in FilesCol
      CFileSize = CStr(CCur(FileItem.Size))
'      CCRC32 = CStr(SBSystem.CRC32(Path & FileItem.Name))
	  CCRC32 = ""
      If SBLocation.AddFile(FileItem.Name, CCRC32, CFileSize, FileItem.Attributes, FileItem.DateLastModified) = False then
        Exit Function
      End If
    Next
  end if
End Function
